@@ -39,7 +39,7 @@ impl<K: Key> Default for Tree<K> {
 
 impl<T: Key> Drop for Tree<T> {
     fn drop(&mut self) {
-        let mut sub_nodes = vec![self.root];
+        let mut sub_nodes = vec![self.root as *const BaseNode];
 
         while !sub_nodes.is_empty() {
             let node = sub_nodes.pop().unwrap();
@@ -70,7 +70,9 @@ impl<T: Key> Drop for Tree<T> {
                     sub_nodes.push(*n);
                 }
             }
-            BaseNode::destroy_node(node);
+            unsafe {
+                std::ptr::drop_in_place(node as *mut BaseNode);
+            }
         }
     }
 }
@@ -85,7 +87,7 @@ impl<T: Key> Tree<T> {
 
     pub fn new() -> Self {
         Tree {
-            root: Node256::new(std::ptr::null(), 0) as *mut BaseNode,
+            root: Node256::new(&[]) as *mut BaseNode,
             _pt_key: PhantomData,
         }
     }
@@ -151,29 +153,24 @@ impl<T: Key> Tree<T> {
 
     pub fn insert(&self, k: T, tid: usize, guard: &Guard) {
         'outer: loop {
-            let mut node: *mut BaseNode = std::ptr::null_mut();
-            let mut next_node = self.root;
-            let mut parent_node: *mut BaseNode;
-
+            let mut parent_node = None;
+            let mut next_node = self.root as *const BaseNode;
             let mut parent_key: u8;
             let mut node_key: u8 = 0;
-            let mut parent_version = 0;
             let mut level = 0;
 
-            loop {
-                parent_node = node;
-                parent_key = node_key;
-                node = next_node;
+            let mut node;
 
-                let mut v = if let Ok(v) = unsafe { &*node }.read_lock() {
+            loop {
+                parent_key = node_key;
+                node = if let Ok(v) = unsafe { &*next_node }.read_lock_n() {
                     v
                 } else {
                     continue 'outer;
                 };
 
                 let mut next_level = level;
-
-                let res = self.check_prefix_pessimistic(unsafe { &*node }, &k, &mut next_level);
+                let res = self.check_prefix_pessimistic(node.as_ref(), &k, &mut next_level);
                 match res {
                     CheckPrefixPessimisticResult::NeedRestart => {
                         continue 'outer;
@@ -181,8 +178,10 @@ impl<T: Key> Tree<T> {
                     CheckPrefixPessimisticResult::Match => {
                         level = next_level;
                         node_key = k.as_bytes()[level as usize];
-                        let next_node_tmp = BaseNode::get_child(node_key, unsafe { &*node });
-                        if unsafe { &*node }.check_or_restart(v).is_err() {
+
+                        let next_node_tmp = BaseNode::get_child(node_key, node.as_ref());
+
+                        if node.check_version().is_err() {
                             continue 'outer;
                         }
 
@@ -194,10 +193,13 @@ impl<T: Key> Tree<T> {
                                     // last key, just insert the tid
                                     BaseNode::set_leaf(tid)
                                 } else {
-                                    let n4 = Node4::new(
-                                        unsafe { k.as_bytes().as_ptr().add(level as usize + 1) },
-                                        k.len() - level as usize - 2,
-                                    );
+                                    let new_prefix = k.as_bytes();
+                                    let n4 =
+                                        Node4::new(&new_prefix[level as usize + 1..k.len() - 1]);
+                                    // let n4 = Node4::new(
+                                    //     unsafe { k.as_bytes().as_ptr().add(level as usize + 1) },
+                                    //     k.len() - level as usize - 2,
+                                    // );
                                     unsafe { &mut *n4 }
                                         .insert(k.as_bytes()[k.len() - 1], BaseNode::set_leaf(tid));
                                     n4 as *mut BaseNode
@@ -206,9 +208,7 @@ impl<T: Key> Tree<T> {
 
                             if BaseNode::insert_and_unlock(
                                 node,
-                                v,
                                 parent_node,
-                                parent_version,
                                 parent_key,
                                 node_key,
                                 new_leaf,
@@ -218,7 +218,7 @@ impl<T: Key> Tree<T> {
                             {
                                 if level as usize != k.len() - 1 {
                                     unsafe {
-                                        Node4::destroy_node(new_leaf as *mut Node4);
+                                        std::ptr::drop_in_place(new_leaf);
                                     }
                                 }
                                 continue 'outer;
@@ -227,58 +227,54 @@ impl<T: Key> Tree<T> {
                             return;
                         };
 
-                        if !parent_node.is_null()
-                            && unsafe { &*parent_node }
-                                .read_unlock(parent_version)
-                                .is_err()
-                        {
-                            continue 'outer;
+                        if let Some(p) = parent_node {
+                            if p.unlock().is_err() {
+                                continue 'outer;
+                            }
                         }
 
                         if BaseNode::is_leaf(next_node) {
-                            if unsafe { &*node }
-                                .upgrade_to_write_lock_or_restart(&mut v)
-                                .is_err()
-                            {
+                            // At this point, the level must point to the last u8 of the key,
+                            // meaning that we are updating an existing value.
+                            let mut write_n = if let Ok(n) = node.upgrade_to_write_lock() {
+                                n
+                            } else {
                                 continue 'outer;
                             };
 
-                            // At this point, the level must point to the last u8 of the key,
-                            // meaning that we are updating an existing value.
                             BaseNode::change(
-                                node,
+                                write_n.as_mut(),
                                 k.as_bytes()[level as usize],
                                 BaseNode::set_leaf(tid),
                             );
 
-                            unsafe { &*node }.write_unlock();
                             return;
                         }
                         level += 1;
-                        parent_version = v;
                     }
 
                     CheckPrefixPessimisticResult::NotMatch((no_match_key, prefix)) => {
-                        if unsafe { &*parent_node }
-                            .upgrade_to_write_lock_or_restart(&mut parent_version)
-                            .is_err()
-                        {
-                            continue 'outer;
-                        }
+                        let mut write_p =
+                            if let Ok(n) = parent_node.unwrap().upgrade_to_write_lock() {
+                                n
+                            } else {
+                                continue 'outer;
+                            };
 
-                        if unsafe { &*node }
-                            .upgrade_to_write_lock_or_restart(&mut v)
-                            .is_err()
-                        {
-                            unsafe { &*parent_node }.write_unlock();
+                        let mut write_n = if let Ok(n) = node.upgrade_to_write_lock() {
+                            n
+                        } else {
                             continue 'outer;
                         };
 
                         // 1) Create new node which will be parent of node, Set common prefix, level to this node
                         let new_node = Node4::new(
-                            unsafe { &*node }.get_prefix().as_ptr(),
-                            (next_level - level) as usize,
+                            &write_n.as_ref().get_prefix()[0..((next_level - level) as usize)],
                         );
+                        // let new_node = Node4::new(
+                        //     write_n.as_ref().get_prefix().as_ptr(),
+                        //     (next_level - level) as usize,
+                        // );
 
                         // 2)  add node and (tid, *k) as children
                         if next_level as usize == k.len() - 1 {
@@ -287,10 +283,14 @@ impl<T: Key> Tree<T> {
                                 .insert(k.as_bytes()[next_level as usize], BaseNode::set_leaf(tid));
                         } else {
                             // otherwise create a new node
-                            let single_new_node = Node4::new(
-                                unsafe { k.as_bytes().as_ptr().add(next_level as usize + 1) },
-                                k.len() - next_level as usize - 2,
-                            );
+                            let single_new_node =
+                                Node4::new(&k.as_bytes()[(next_level as usize + 1)..k.len() - 1]);
+                            // k.len() - next_level as usize - 2,
+                            // let single_new_node = Node4::new(
+                            //     unsafe { k.as_bytes().as_ptr().add(next_level as usize + 1) },
+                            //     k.len() - next_level as usize - 2,
+                            // );
+
                             unsafe { &mut *single_new_node }
                                 .insert(k.as_bytes()[k.len() - 1], BaseNode::set_leaf(tid));
                             unsafe { &mut *new_node }.insert(
@@ -299,23 +299,21 @@ impl<T: Key> Tree<T> {
                             );
                         }
 
-                        unsafe { &mut *new_node }.insert(no_match_key, node);
+                        unsafe { &mut *new_node }.insert(no_match_key, write_n.as_mut());
 
                         // 3) upgradeToWriteLockOrRestart, update parentNode to point to the new node, unlock
-                        BaseNode::change(parent_node, parent_key, new_node as *mut BaseNode);
-                        unsafe { &*parent_node }.write_unlock();
+                        BaseNode::change(write_p.as_mut(), parent_key, new_node as *mut BaseNode);
 
                         // 4) update prefix of node, unlock
-                        let mut_node = unsafe { &mut *node };
-                        let prefix_len = mut_node.get_prefix_len();
-                        mut_node.set_prefix(
+                        let prefix_len = write_n.as_ref().get_prefix_len();
+                        write_n.as_mut().set_prefix(
                             prefix.as_ptr(),
                             (prefix_len - (next_level - level + 1)) as usize,
                         );
-                        mut_node.write_unlock();
                         return;
                     }
                 }
+                parent_node = Some(node);
             }
         }
     }
@@ -398,30 +396,29 @@ impl<T: Key> Tree<T> {
         range_scan.scan()
     }
 
+    #[allow(clippy::unnecessary_unwrap)]
     pub fn remove(&self, k: &T, guard: &Guard) {
         'outer: loop {
-            let mut node: *mut BaseNode = std::ptr::null_mut();
-            let mut next_node = self.root;
-            let mut parent_node: *mut BaseNode;
+            let mut next_node = self.root as *const BaseNode;
+            let mut parent_node: Option<ReadGuard> = None;
 
             let mut parent_key: u8;
             let mut node_key: u8 = 0;
-            let mut parent_version = 0;
             let mut level = 0;
             let mut key_tracker = KeyTracker::default();
 
-            loop {
-                parent_node = node;
-                parent_key = node_key;
-                node = next_node;
+            let mut node;
 
-                let mut v = if let Ok(v) = unsafe { &*node }.read_lock() {
+            loop {
+                parent_key = node_key;
+
+                node = if let Ok(v) = unsafe { &*next_node }.read_lock_n() {
                     v
                 } else {
                     continue 'outer;
                 };
 
-                match Self::check_prefix(unsafe { &*node }, k, level) {
+                match Self::check_prefix(node.as_ref(), k, level) {
                     CheckPrefixResult::NotMatch => {
                         return;
                     }
@@ -432,9 +429,9 @@ impl<T: Key> Tree<T> {
                         level = l;
                         node_key = k.as_bytes()[level as usize];
 
-                        let next_node_tmp = BaseNode::get_child(node_key, unsafe { &*node });
+                        let next_node_tmp = BaseNode::get_child(node_key, node.as_ref());
 
-                        if unsafe { &*node }.check_or_restart(v).is_err() {
+                        if node.check_version().is_err() {
                             continue 'outer;
                         };
 
@@ -455,48 +452,43 @@ impl<T: Key> Tree<T> {
                                 return;
                             }
 
-                            if !parent_node.is_null() && unsafe { &*node }.get_count() == 1 {
-                                if unsafe { &*parent_node }
-                                    .upgrade_to_write_lock_or_restart(&mut parent_version)
-                                    .is_err()
-                                {
+                            if parent_node.is_some() && node.as_ref().get_count() == 1 {
+                                let mut write_p =
+                                    if let Ok(p) = parent_node.unwrap().upgrade_to_write_lock() {
+                                        p
+                                    } else {
+                                        continue 'outer;
+                                    };
+
+                                let mut write_n = if let Ok(n) = node.upgrade_to_write_lock() {
+                                    n
+                                } else {
                                     continue 'outer;
-                                }
+                                };
 
-                                if unsafe { &*node }
-                                    .upgrade_to_write_lock_or_restart(&mut v)
-                                    .is_err()
-                                {
-                                    unsafe { &*parent_node }.write_unlock();
-                                    continue 'outer;
-                                }
+                                BaseNode::remove_key(write_p.as_mut(), parent_key);
 
-                                BaseNode::remove_key(parent_node, parent_key);
-                                unsafe { &*parent_node }.write_unlock();
-
-                                unsafe { &*node }.write_unlock_obsolete();
-                                let d_n = unsafe { &mut *node };
-                                guard.defer(move || {
-                                    BaseNode::destroy_node(d_n);
+                                write_n.mark_obsolete();
+                                guard.defer(move || unsafe {
+                                    std::ptr::drop_in_place(write_n.as_mut());
                                 });
                             } else {
-                                debug_assert!(!parent_node.is_null());
-                                if unsafe { &*node }
-                                    .upgrade_to_write_lock_or_restart(&mut v)
-                                    .is_err()
-                                {
+                                debug_assert!(parent_node.is_some());
+                                let mut write_n = if let Ok(n) = node.upgrade_to_write_lock() {
+                                    n
+                                } else {
                                     continue 'outer;
-                                }
-                                BaseNode::remove_key(node, node_key);
-                                unsafe { &*node }.write_unlock();
+                                };
+
+                                BaseNode::remove_key(write_n.as_mut(), node_key);
                             }
                             return;
                         }
                         level += 1;
                         key_tracker.push(node_key);
-                        parent_version = v;
                     }
                 }
+                parent_node = Some(node);
             }
         }
     }
